const encodeIPFSUri = (cid) => { 
  try {
    const decodedCID = bs58.decode(cid);
    const slicedCID = decodedCID.slice(2);
    const uint8Array = new Uint8Array(slicedCID);
    let hex = '0x';
    for (let i = 0; i < uint8Array.length; i++) 
      hex += uint8Array[i].toString(16).padStart(2, '0');
    return hex;
  } catch (e) {
    return "0x0000000000000000000000000000000000000000000000000000000000000000"; 
  }
}

const encodeIPFSUriFrom = async (prefix, itemId) => {
  // Create the content's encoded IPFS URL.
  const cidUrl = `${prefix}${itemId}/nft.json.cid.txt`;
  const cid = await (await fetch(cidUrl)).text();
  return cid ? encodeIPFSUri(cid) : "0x0000000000000000000000000000000000000000000000000000000000000000";
}

const formatDate = (date) => {
  const month = date.getMonth() + 1;
  const day = date.getDate();
  const year = date.getFullYear();

  return `${month}.${day}.${year.toString().slice(-2)}`;
}

const formatTimestamp = (timestamp) => {
  const months = [
    'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'
  ];

  const date = new Date((timestamp + 978307200) * 1000);
  return formatDate(date);
}

const formatPlayclock = (seconds) => {
  const minutes = Math.floor(seconds / 60);
  seconds = Math.floor(seconds % 60);
  return minutes + ':' + (seconds < 10 ? '0' : '') + seconds;
} 

const resolveChainId = (value) => {
  switch (value) {
    case "ethereum sepolia": return 11155111; 
    case "optimism sepolia": return 11155420; 
  }
}

const resolveChainSelectIndex = (chain) => {
  switch (chain) {
    case "ethereum sepolia": return 0; 
    case "optimism sepolia": return 1; 
  }
}

const resolveChain = (chainId) => {
  switch (chainId) {
    case 11155111: return "ethereum sepolia"; 
    case 11155420: return "optimism sepolia"; 
  }
}

const cpnProjectId = (chain) => {
  switch (chain) {
    case "ethereum sepolia":
      return 1;
    case "optimism sepolia":
      return 1;
  }
}

const resolveCollectionId = (chain) => {
  switch (chain) {
    case "ethereum sepolia":
      return env.ethereumSepoliaCollectionID || cpnProjectId(chain);
    case "optimism sepolia":
      return env.optimismSepoliaCollectionID || cpnProjectId(chain);
  }
}

const resolveCollectionCategory = (chain) => {
  switch (chain) {
    case "ethereum sepolia":
      return env.ethereumSepoliaCollectionCategory || 0;
    case "optimism sepolia":
      return env.optimismSepoliaCollectionCategory || 0;
  }
}

const resolveCPNBeneficiaryAddress = (chain) => {
  switch (chain) {
    case "ethereum sepolia":
      return env.ethereumSepoliaCPNBeneficiaryAddress;
    case "optimism sepolia":
      return env.optimismSepoliaCPNBeneficiaryAddress;
  }
}

const projectLinkBase = (chain) => {
  switch (chain) {
    case "ethereum sepolia":
      return "https://juicebox.money/v2/p/";
    case "optimism sepolia":
      return "https://goerli.juicebox.money/v2/p/";
  }
}

const renderMarkdown = (text) => {
  const md = window.markdownit({
    html: true,
    xhtmlOut: false,
    linkify: true
  });
  md.use(window.markdownItAnchor, { permalink: true, permalinkBefore: false, permalinkSymbol: '#' })
  .use(window.markdownitTaskLists)
  .use(window.markdownitFootnote);
  return md.render(text);
}

const fixedPointNumber = (numberString, decimals) => {
  // Find the position of the decimal point
  let decimalPos = numberString.indexOf('.');
  let fixedString;

  if (decimalPos !== -1) {
    // Split the number into whole and decimal parts
    let wholePart = numberString.substring(0, decimalPos);
    let decimalPart = numberString.substring(decimalPos + 1);
    // If the decimal part is shorter than the desired precision, pad it with zeros
    while (decimalPart.length < decimals) {
      decimalPart += '0';
    }
    // Combine the whole part and the truncated/padded decimal part
    fixedString = wholePart + decimalPart.substring(0, decimals);
  } else {
    // If there is no decimal part, just pad the number with zeros
    fixedString = numberString + '0'.repeat(decimals);
  }

  // Convert the string to a BigInt
  return BigInt(fixedString);
}

const loadingAnimationPace = 100;
let animationIntervals = {};

/// Animate the loading ticker.
const startLoadingAnimation = (loadingAnimationId) => {
  const loadingAnimation = document.getElementById(loadingAnimationId);
  loadingAnimation.style.display = "initial";
  const loadingAnimationFrames = ['-', '\\', '|', '/'];
  let currentFrame = 0;

  const animate = (firstTime) => {
    loadingAnimation.textContent = loadingAnimationFrames[currentFrame];
    currentFrame = (currentFrame + 1) % loadingAnimationFrames.length;
    if (firstTime || animationIntervals[loadingAnimationId]) animationIntervals[loadingAnimationId] = setTimeout(() => animate(false), loadingAnimationPace);
  }

  animate(true);
}

/// Stop animating the loading ticker.
const stopLoadingAnimation = (loadingAnimationId) => {
  const loadingAnimation = document.getElementById(loadingAnimationId);
  animationIntervals[loadingAnimationId] = clearTimeout(animationIntervals[loadingAnimationId]);
  loadingAnimation.style.display = "none";
}

let audioAnimationInterval;
const audioAnimationFrames = ['▄', '▅', '█'];

const prepareAudioAnimation = (audioAnimationId) => {
  const audioAnimation = document.getElementById(audioAnimationId);
  audioAnimation.innerHTML = audioAnimationFrames[1] + audioAnimationFrames[2] + audioAnimationFrames[0];
}
const startAudioAnimation = (audioAnimationId) => {
  const audioAnimation = document.getElementById(audioAnimationId);
  const audioAnimationPace = 100;

  const animate = () => {
    const oneOrTwo = Math.floor(Math.random() * 2) + 1;
    const char0 = audioAnimationFrames[(audioAnimationFrames.indexOf(audioAnimation.innerHTML.charAt(0)) + oneOrTwo) % 3]; 
    const char1 = audioAnimationFrames[(audioAnimationFrames.indexOf(audioAnimation.innerHTML.charAt(1)) + oneOrTwo) % 3]; 
    const char2 = audioAnimationFrames[(audioAnimationFrames.indexOf(audioAnimation.innerHTML.charAt(2)) + oneOrTwo) % 3]; 
    audioAnimation.innerHTML = char0 + char1 + char2;
    animationIntervals[audioAnimationId] = setTimeout(animate, audioAnimationPace);
  }
  animate();
}

const stopAudioAnimation = (audioAnimationId) => {
  const audioAnimation = document.getElementById(audioAnimationId);
  animationIntervals[audioAnimationId] = clearTimeout(animationIntervals[audioAnimationId]);
}

